c@***h* CMORPH-CDR/a1.CMORPH_V1.0_CRT.get_ADJ_8km-30min.f
c
c=======================================================================
c PROGRAM
c   a1.CMORPH_V1.0_CRT.get_ADJ_8km-30min.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 8km-30min bias-corrected CMORPH data using 0.25deg-HLY
c   smoothed adjust factors over globe and 8km-30min raw CMORPH data
c
c-----------------------------------------------------------------------
c USAGE
c   a1.CMORPH_V1.0_CRT.get_ADJ_8km-30min.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   FCT_SMT_0.25deg-HLY_<yyyymmdd>         : 0.25deg-HLY smoothed adjust
c                                            factors daily file
c   CMORPH_V1.0_RAW_8km-30min_<yyyymmddhh> : 8km-30min raw CMORPH data
c                                            hourly files
c
c OUTPUT FILES
c   CMORPH_V1.0_ADJ_8km-30min_<yyyymmddhh> : 8km-30min bias-corrected
c                                            CMORPH data hourly files
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX         : X-dimension (0-360) in 0.25deg grid
c   NY         : Y-dimension (60S-60N) in 0.25deg grid
c   NX8        : X-dimension (0-360) in 8km grid
c   NY8        : Y-dimension (60S-60N) in 8km grid
c   LON0       : longitude of center point of box (1,1) in 8km grid
c   LAT0       : latitude of center point of box (1,1) in 8km grid
c   DLON       : X-increment in 8km grid
c   DLAT       : Y-increment in 8km grid
c   -----------:------------------------
c   yyyymmdd   : target date
c   f_fct      : file name of 0.25deg-HLY smoothed adjust factors over globe
c   f_raw      : 8km-30min raw CMORPH data file name
c   f_adj      : 8km-30min bias-corrected CMORPH data file name
c   yyyy,mm,dd : target date
c   hh         : loop counter for hours
c   hf         : loop counter for half-hour
c   ii         : loop counter in X-direction
c   jj         : loop counter in Y-direction
c   is         : X-index of working 8km grid box in 0.25deg grid
c   js         : Y-index of working 8km grid box in 0.25deg grid
c   iilon()    : X-index of 8km grid boxes in 0.25deg grid
c   jjlat()    : Y-index of 8km grid boxes in 0.25deg grid
c   lon        : longitude of working 8km grid box
c   lat        : latitude of working 8km grid box
c   fct()      : 0.25deg-HLY adjust factors
c   raw()      : 8km-30min raw CMORPH data
c   adj()      : 8km-30min bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'correct_bias_8km_30min.lnx.f'
c
c   step 1 of 6 to generate CMORPH_V1.0_CRT dataset
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program a1_CMOPRH_V10_CRT_get_ADJ_8km_30min
      implicit none

      integer NX, NY, NX8, NY8
      real LON0, DLON, LAT0, DLAT
      parameter( NX=1440, NY=480, NX8=4948, NY8=1649 )
      parameter( LON0=0.036378335, DLON=0.072756669 )
      parameter( LAT0=-59.963614,  DLAT=0.072771377 )

      character yyyymmdd*8, f_fct*36, f_raw*44, f_adj*44
      integer yyyy, mm, dd, hh, hf, ii, jj, is, js
      integer iilon(NX8), jjlat(NY8), tt
      real lon, lat, fct(NX,NY), raw(NX8,NY8), adj(NX8,NY8)

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c generate index of 8km grid boxes in 0.25deg grid
c-----------------------------------------------------------------------
      do jj=1, NY8
        lat=LAT0+(jj-1)*DLAT
        jjlat(jj)=int((lat+60.0)/0.25)+1
        if( jjlat(jj).lt.1.or.jjlat(jj).gt.NY ) then
          write(*, *) '#### wrong latitude: ', jj, lat, jjlat(jj)
          stop
        endif
      enddo
      do ii=1, NX8
        lon=LON0+(ii-1)*DLON
        iilon(ii)=int((lon-0.0)/0.25)+1
        if( iilon(ii).lt.1.or.iilon(ii).gt.NX ) then
          write(*, *) '#### wrong longitude: ', ii, lon, iilon(ii)
          stop
        endif
      enddo

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|1234<--
      f_raw='../work/CMORPH_V1.0_RAW_8km-30min_yyyymmddhh'
      f_fct='../work/FCT_SMT_0.25deg-HLY_yyyymmdd'
      f_adj='../work/CMORPH_V1.0_ADJ_8km-30min_yyyymmddhh'

      if( iargc().ge.2 ) then
        f_raw(4:7)='redo'
        f_fct(4:7)='redo'
        f_adj(4:7)='redo'
      endif

      write(f_fct(29:36), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c open 0.25deg-HLY adjust factors daily file for input
c-----------------------------------------------------------------------
      open(10, file=f_fct, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_fct

c-----------------------------------------------------------------------
c perform bias correction for 2 half-hour periods in each hour
c-----------------------------------------------------------------------
      do hh=1, 24

        write(f_raw(35:44), '(i4.4,3i2.2)') yyyy, mm, dd, hh-1
        write(f_adj(35:44), '(i4.4,3i2.2)') yyyy, mm, dd, hh-1

        open(11, file=f_raw, status='old',
     1      access='direct', recl=NX8*NY8*4)
        write(*, *) ' <--  ', f_raw
        open(20, file=f_adj, access='direct', recl=NX8*NY8*4)

        read(10, rec=hh) fct
        do hf=1, 2
          read(11, rec=hf) raw
          do jj=1, NY8
            do ii=1, NX8
              js=jjlat(jj)
              is=iilon(ii)
              if( raw(ii,jj).ge.0.0.and.fct(is,js).ge.0.0 ) then
                if( fct(is,js).gt.5.0 ) fct(is,js)=5.0
                adj(ii,jj)=raw(ii,jj)*fct(is,js)
              else
                adj(ii,jj)=-999.0
              endif
            enddo
          enddo
c-----------------------------------------------------------------------
c truncate to 2 digits after decimal point
c-----------------------------------------------------------------------
          do jj=1, NY8
            do ii=1, NX8
              if( adj(ii,jj).le.-999.0 ) then
                adj(ii,jj)=-999.0
              else
                tt=nint(adj(ii,jj)*100.0)
                adj(ii,jj)=0.01*tt
              endif
            enddo
          enddo
          write(20, rec=hf) adj
        enddo
        close(11)
        close(20)
        write(*, *) '  ==> ', f_adj

      enddo

      close(10)

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
