c@***h* CMORPH-CDR/a2.CMORPH_V1.0_CRT.get_ADJ_0.25deg-30min.f
c
c=======================================================================
c PROGRAM
c   a2.CMORPH_V1.0_CRT.get_ADJ_0.25deg-30min.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 0.25deg-30min bias-corrected CMORPH data from 8km-30min
c   bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c USAGE
c   a2.CMORPH_V1.0_CRT.get_ADJ_0.25deg-30min.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_ADJ_8km-30min_<yyyymmddhh> : 8km-30min bias-corrected
c                                            CMORPH data hourly files
c
c OUTPUT FILES
c   CMORPH_V1.0_ADJ_0.25deg-30min_<yyyymmdd> : 0.25deg-30min bias-corrected
c                                              CMORPH data daily file
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX         : X-dimension (0-360) in 0.25deg grid
c   NY         : Y-dimension (60S-60N) in 0.25deg grid
c   NX8        : X-dimension (0-360) in 8km grid
c   NY8        : Y-dimension (60S-60N) in 8km grid
c   LON0       : longitude of center point of box (1,1) in 8km grid
c   LAT0       : latitude of center point of box (1,1) in 8km grid
c   DLON       : X-increment in 8km grid
c   DLAT       : Y-increment in 8km grid
c   -----------:------------------------
c   yyyymmdd   : target date
c   f_8km      : 8km-30min bias-corrected CMORPH data file name
c   f_025      : 0.25deg-30min bias-corrected CMORPH data file name
c   yyyy,mm,dd : target date
c   hh         : loop counter for hours
c   hf         : loop counter for half-hour
c   ii         : loop counter in X-direction
c   jj         : loop counter in Y-direction
c   is         : X-index of working 8km grid box in 0.25deg grid
c   js         : Y-index of working 8km grid box in 0.25deg grid
c   iilon()    : X-index of 8km grid boxes in 0.25deg grid
c   jjlat()    : Y-index of 8km grid boxes in 0.25deg grid
c   ct()       : count of valid data points
c   lon        : longitude of working 8km grid box
c   lat        : latitude of working 8km grid box
c   rain8()    : 8km-30min bias-corrected CMORPH data
c   ss()       : 0.25deg-30min bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'CMORPH_def_0.25deg_30min.lnx.f'
c
c   step 2 of 6 to generate CMORPH_V1.0_CRT dataset
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program a2_CMORPH_V10_CRT_get_ADJ_025deg_30min
      implicit none

      integer NX, NY, NX8, NY8
      real LON0, DLON, LAT0, DLAT
      parameter( NX=1440, NY=480, NX8=4948, NY8=1649 )
      parameter( LON0=0.036378335, DLON=0.072756669 )
      parameter( LAT0=-59.963614,  DLAT=0.072771377 )

      character yyyymmdd*8, f_8km*44, f_025*46
      integer yyyy, mm, dd, hh, hf, ii, jj, is, js
      integer iilon(NX8), jjlat(NY8), ct(NX,NY), tt
      real lon, lat, rain8(NX8,NY8), ss(NX,NY)

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c generate index of 8km grid boxes in 0.25deg grid
c-----------------------------------------------------------------------
      do jj=1, NY8
        lat=LAT0+(jj-1)*DLAT
        jjlat(jj)=int((lat+60.0)/0.25)+1
        if( jjlat(jj).lt.1.or.jjlat(jj).gt.NY ) then
          write(*, *) '#### wrong latitude: ', jj, lat, jjlat(jj)
          stop
        endif
      enddo
      do ii=1, NX8
        lon=LON0+(ii-1)*DLON
        iilon(ii)=int((lon-0.0)/0.25)+1
        if( iilon(ii).lt.1.or.iilon(ii).gt.NX ) then
          write(*, *) '#### wrong longitude: ', ii, lon, iilon(ii)
          stop
        endif
      enddo

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|123456<--
      f_8km='../work/CMORPH_V1.0_ADJ_8km-30min_yyyymmddhh'
      f_025='../work/CMORPH_V1.0_ADJ_0.25deg-30min_yyyymmdd'

      if( iargc().ge.2 ) then
        f_8km(4:7)='redo'
        f_025(4:7)='redo'
      endif

      write(f_025(39:46), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c open 0.25deg-30min bias-corrected CMORPH daily file for output
c-----------------------------------------------------------------------
      open(20, file=f_025, access='direct', recl=NX*NY*4)

c-----------------------------------------------------------------------
c generate 0.25deg-30min bias-corrected CMORPH data for 2 half-hour
c periods in each hour
c-----------------------------------------------------------------------
      do hh=1, 24

        write(f_8km(35:44), '(i4.4,3i2.2)') yyyy, mm, dd, hh-1
        open(10, file=f_8km, status='old',
     1      access='direct', recl=NX8*NY8*4)
        write(*, *) ' <--  ', f_8km

        do hf=1, 2
          read(10, rec=hf) rain8
          do jj=1, NY
            do ii=1, NX
              ct(ii,jj)=0
              ss(ii,jj)=0.0
            enddo
          enddo
          do js=1, NY8
            do is=1, NX8
              jj=jjlat(js)
              ii=iilon(is)
              if( rain8(is,js).ge.0.0 ) then
                ct(ii,jj)=ct(ii,jj)+1
                ss(ii,jj)=ss(ii,jj)+rain8(is,js)
              endif
            enddo
          enddo
          do jj=1, NY
            do ii=1, NX
              if( ct(ii,jj).gt.0 ) then
                ss(ii,jj)=ss(ii,jj)/ct(ii,jj)
              else
                ss(ii,jj)=-999.0
              endif
            enddo
          enddo
c-----------------------------------------------------------------------
c truncate to 2 digits after decimal point
c-----------------------------------------------------------------------
          do jj=1, NY
            do ii=1, NX
              if( ss(ii,jj).le.-999.0 ) then
                ss(ii,jj)=-999.0
              else
                tt=nint(ss(ii,jj)*100.0)
                ss(ii,jj)=0.01*tt
              endif
            enddo
          enddo
          write(20, rec=(hh-1)*2+hf) ss
        enddo

        close(10)

      enddo

      close(20)
      write(*, *) '  ==> ', f_025

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
