c@***h* CMORPH-CDR/o3.CMORPH_V1.0_CRT.get_GPCP_ADJ_2.50deg-PEN.f
c
c=======================================================================
c PROGRAM
c   o3.CMORPH_V1.0_CRT.get_GPCP_ADJ_2.50deg-PEN.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 2.50deg-PEN (60S-60N) GPCP data with land/ocean mixed
c   grid boxes processed
c
c-----------------------------------------------------------------------
c USAGE
c   o3.CMORPH_V1.0_CRT.get_GPCP_ADJ_2.50deg-PEN.x <yyyy> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx : 0.25deg land mask
c   CMORPH_V1.0_LAND_MASK_2.50deg.lnx : 2.50deg land mask
c   CMORPH_0.25deg-PEN_<yyyypp>       : 0.25deg-PEN CMORPH data
c   GPCP_RAW_2.50deg-PEN_<yyyypp>     : 2.50deg-PEN (90S-90N) raw GPCP data
c
c OUTPUT FILES
c   GPCP_ADJ_2.50deg-PEN_<yyyypp> : 2.50deg-PEN (60S-60N) adjusted GPCP data
c                                   (with land/ocean mixed grid boxes processed)
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyy : target year
c   redo : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX        : X-dimension (0-360) in 0.25deg grid
c   NY        : Y-dimension (60S-60N) in 0.25deg grid
c   NX2       : X-dimension (0-360) in 2.50deg grid
c   NY2       : Y-dimension (60S-60N) in 2.50deg grid
c   NY20      : Y-dimension (90S-90N) in 2.50deg grid
c   NP        : number of pentad in a yearc
c   ----------:----------------------------
c   yyyy      : target year
c   f_mk0     : 0.25deg land mask file name
c   f_mk2     : 2.50deg land mask file name
c   f_cmp     : 0.25deg-PEN CMORPH data file name
c   f_raw     : 2.50deg-PEN (90S-90N) raw GPCP data file name
c   f_adj     : 2.50deg-PEN (60S-60N) adjusted GPCP data file name
c   mask0()   : 0.25deg land mask
c   mask2()   : 2.50deg land mask
c   cmp()     : 0.25deg-PEN CMORPH data
c   raw()     : 2.50deg-PEN (90S-90N) raw GPCP data
c   adj()     : 2.50deg-PEN (60S-60N) adjusted GPCP data
c   pp        : loop counter for pentad
c   ii,i      : loop counters in X-direction
c   i1,i2     : X-boundaries of 0.25deg grid boxes in a 2.50deg grid box
c   jj,j      : loop counters in Y-direction
c   j1,j2     : Y-boundaries of 0.25deg grid boxes in a 2.50deg grid box
c   c_ttl     : count of valid 0.25deg CMORPH data in a 2.50deg grid box
c   c_ocn     : count of valid 0.25deg CMORPH data over ocean
c   s_ttl     : sum of valid 0.25deg CMORPH data in a 2.50deg grid box
c   s_ocn     : sum of valid 0.25deg CMORPH data over ocean
c   r_ttl     : average CMORPH data in a 2.50deg grid box
c   r_ocn     : average CMORPH data over ocean
c   ff        : adjust factor
c   cmp_there : check if CMOPRH data file exists
c   raw_there : check if raw GPCP data file exists
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'def_gpcp_pen_board.lnx.f'
c
c   step 3 of 6 to generate 0.25deg-DLY adjust factors over ocean
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program o3_CMORPH_V10_CRT_get_GPCP_ADJ_250deg_PEN
      implicit none

      integer NX, NY, NX2, NY2, NY20, NP
      parameter( NX=1440, NY=480, NX2=144, NY2=48, NY20=72, NP=73 )

      character yyyy*4, f_mk0*42,  f_mk2*42
      character f_cmp*33, f_raw*35, f_adj*35
      real mask0(NX,NY), mask2(NX2,NY2)
      real cmp(NX,NY), raw(NX2,NY20), adj(NX2,NY2)
      integer pp, ii, jj, i, i1, i2, j, j1, j2, c_ttl, c_ocn
      real s_ttl, s_ocn, r_ttl, r_ocn, ff
      logical cmp_there, raw_there

c-----------------------------------------------------------------------
c get target year
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '######################################'
        write(*, *) '#                                    #'
        write(*, *) '#  parameter <yyyy> [<redo>] needed  #'
        write(*, *) '#                                    #'
        write(*, *) '######################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyy)

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|12<--
      f_mk0='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_mk2='../table/CMORPH_V1.0_LAND_MASK_2.50deg.lnx'
      f_cmp='../work/CMORPH_0.25deg-PEN_yyyypp'
      f_raw='../work/GPCP_RAW_2.50deg-PEN_yyyypp'
      f_adj='../work/GPCP_ADJ_2.50deg-PEN_yyyypp'

      if( iargc().ge.2 ) then
        f_cmp(4:7)='redo'
        f_raw(4:7)='redo'
        f_adj(4:7)='redo'
      endif

      write(f_cmp(28:31), '(a4)') yyyy

      write(f_raw(30:33), '(a4)') yyyy

      write(f_adj(30:33), '(a4)') yyyy

c-----------------------------------------------------------------------
c read in 0.25deg and 2.50deg land masks
c-----------------------------------------------------------------------
      open(10, file=f_mk0, status='old',
     1    access='direct', recl=NX*NY*4)
      open(11, file=f_mk2, status='old',
     1    access='direct', recl=NX2*NY2*4)
      write(*, *) ' <--  ', f_mk0
      write(*, *) ' <--  ', f_mk2
      read(10, rec=1) mask0
      read(11, rec=1) mask2
      close(10)
      close(11)

c-----------------------------------------------------------------------
c repeat for each pentad in the year
c-----------------------------------------------------------------------
      do pp=1, NP

        write(f_cmp(32:33), '(i2.2)') pp
        write(f_raw(34:35), '(i2.2)') pp
        write(f_adj(34:35), '(i2.2)') pp

c-----------------------------------------------------------------------
c check if CMOPRH and raw GPCP files exist
c-----------------------------------------------------------------------
        inquire(file=f_cmp, exist=cmp_there)
        inquire(file=f_raw, exist=raw_there)
        if( cmp_there.and.raw_there ) then

c-----------------------------------------------------------------------
c read in 0.25deg CMORPH data and 2.50deg (90S-90N) raw GPCP data
c-----------------------------------------------------------------------
          open(12, file=f_cmp, status='old',
     1        access='direct', recl=NX*NY*4)
          open(13, file=f_raw, status='old',
     1        access='direct', recl=NX2*NY20*4)
          write(*, *) ' <--  ', f_cmp
          write(*, *) ' <--  ', f_raw
          read(12, rec=1) cmp
          read(13, rec=1) raw
          close(12)
          close(13)

c-----------------------------------------------------------------------
c extract (60S-60N) GPCP data and adjust for land/ocean mixed grid boxes
c-----------------------------------------------------------------------
          do jj=1, NY2
            do ii=1, NX2
              if( raw(ii,jj+12).lt.0.0 ) then
                raw(ii,jj+12)=-999.0
              endif
              if( mask2(ii,jj).ne.0.0 ) then
                adj(ii,jj)=-999.0
              else
                if( raw(ii,jj+12).lt.0.0 ) then
                  adj(ii,jj)=-999.0
                else if( raw(ii,jj+12).eq.0.0 ) then
                  adj(ii,jj)=0.0
                else
                  c_ttl=0
                  s_ttl=0.0
                  c_ocn=0
                  s_ocn=0.0
                  j1=(jj-1)*10+1
                  j2=j1+9
                  i1=(ii-1)*10+1
                  i2=i1+9
                  do j=j1, j2
                    do i=i1, i2
                      if( cmp(i,j).ge.0.0 ) then
                        c_ttl=c_ttl+1
                        s_ttl=s_ttl+cmp(i,j)
                        if( mask0(i,j).eq.0.0 ) then
                          c_ocn=c_ocn+1
                          s_ocn=s_ocn+cmp(i,j)
                        endif
                      endif
                    enddo
                  enddo
                  if( c_ttl.eq.0.or.c_ocn.eq.0 ) then
                    adj(ii,jj)=raw(ii,jj+12)
                  else
                    r_ttl=s_ttl/c_ttl
                    r_ocn=s_ocn/c_ocn
                    if( r_ttl.eq.0.0 ) then
                      adj(ii,jj)=raw(ii,jj+12)
                    else
                      ff=r_ocn/r_ttl
                      if( ff.lt.0.5 ) ff=0.5
                      if( ff.gt.2.0 ) ff=2.0
                      adj(ii,jj)=raw(ii,jj+12)*ff
                    endif
                  endif
                endif
              endif
            enddo
          enddo

c-----------------------------------------------------------------------
c output 2.50deg (60S-60N) adjusted GPCP data
c-----------------------------------------------------------------------
          open(20, file=f_adj, access='direct', recl=NX2*NY2*4)
          write(20, rec=1) adj
          close(20)
          write(*, *) '  ==> ', f_adj

        endif

      enddo

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
