c@***h* CMORPH-CDR/o4.CMORPH_V1.0_CRT.get_FCT_OCN_2.50deg-PEN.f
c
c=======================================================================
c PROGRAM
c   o4.CMORPH_V1.0_CRT.get_FCT_OCN_2.50deg-PEN.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 2.50deg-PEN adjust factors over ocean using 0.25deg-PEN
c   adjusted GPCP data and CMORPH data for a 19-pentad processing window
c   centering at target pentad
c
c-----------------------------------------------------------------------
c USAGE
c   o4.CMORPH_V1.0_CRT.get_FCT_OCN_2.50deg-PEN.x <yyyypp> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_2.50deg.lnx : 2.50deg land mask
c   GPCP_ADJ_2.50deg-PEN_<yyyypp>     : 2.50deg-PEN adjusted GPCP data
c   CMORPH_2.50deg-PEN_<yyyypp>       : 2.50deg-PEN CMORPH data
c
c OUTPUT FILES
c   FCT_OCN_2.50deg-PEN_<yyyypp> : 2.50deg-PEN adjust factors over ocean
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyypp : target pentad
c   redo   : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX        : X-dimension (0-360) in 2.50deg grid
c   NY        : Y-dimension (60S-60N) in 2.50deg grid
c   NP        : number of pentad in a year
c   WIN       : half of processing window
c   MIN_PAIR  : minimum number of data pairs required
c   MIN_RAIN  : minimum number of raining pairs required
c   ----------:-----------------------------------------
c   yyyypp    : target pentad
c   f_msk     : 2.50deg land mask file name
c   f_adj     : 2.50deg-PEN adjusted GPCP data file name
c   f_cmp     : 2.50deg-PEN CMORPH data file name
c   f_fct     : file name of 2.50deg-PEN adjust factors over ocean
c   yyyy,pp   : target pentad
c   year,pent : working pentad
c   k         : loop counter for pentad
c   dis       : searching distance
c   dis2      : distance^2 between working and target grid boxes
c   dis2max   : maximum dis2
c   ii,i      : loop counters in X-direction
c   i1,i2     : X-boundaries for searching window
c   is        : X-index of working grid box
c   jj,j      : loop counters in Y-direction
c   j1,j2     : Y-boundaries for searching window
c   js        : Y-index of working grid box
c   mask()    : 2.50deg land mask
c   adj()     : 2.50deg-PEN adjusted GPCP data
c   cmp()     : 2.50deg-PEN CMORPH data
c   fct()     : 2.50deg-PEN adjust factors over ocean
c   sc()      : sum of CMOPRH data over processing window
c   sa()      : sum of adjusted GPCP data over processing window
c   sw()      : sum of weight over processing window
c   wt        : weight over processing window
c   wd        : weight over searching window
c   ssc       : sum of sc() over searching window
c   ssa       : sum of sa() over searching window
c   ssw       : sum of weight over searching window
c   cp()      : count of data pairs over processing window
c   cr()      : count of raining pairs over processing window
c   ccp       : sum of cp() over searching window
c   ccr       : sum of cr() over searching window
c   cmp_there : check if CMORPH data file exists
c   adj_there : check if adjusted GPCP data file exists
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'def_adj_factor_2.50deg_pen_ocean.lnx.f'
c
c   step 4 of 6 to generate 0.25deg-DLY adjust factors over ocean
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program o4_CMORPH_V10_CRT_get_FCT_OCN_250deg_PEN
      implicit none

      integer NX, NY, NP, WIN, MIN_PAIR, MIN_RAIN
      parameter( NX=144, NY=48, NP=73, WIN=9, MIN_PAIR=80, MIN_RAIN=50 )

      character yyyypp*6, f_msk*42, f_adj*35, f_cmp*33, f_fct*34
      integer yyyy, pp, year, pent, k, dis, dis2, dis2max
      integer ii, jj, i, i1, i2, is, j, j1, j2, js
      real mask(NX,NY), adj(NX,NY), cmp(NX,NY), fct(NX,NY)
      real sc(NX,NY), sa(NX,NY), sw(NX,NY), wt, wd, ssc, ssa, ssw
      integer cp(NX,NY), cr(NX,NY), ccp, ccr
      logical cmp_there, adj_there

c-----------------------------------------------------------------------
c get target pentad
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '########################################'
        write(*, *) '#                                      #'
        write(*, *) '#  parameter <yyyypp> [<redo>] needed  #'
        write(*, *) '#                                      #'
        write(*, *) '########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyypp)
      read(yyyypp, '(i4.4,i2.2)') yyyy, pp

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|123456789|1234<--
      f_msk='../table/CMORPH_V1.0_LAND_MASK_2.50deg.lnx'
      f_adj='../work/GPCP_ADJ_2.50deg-PEN_yyyypp'
      f_cmp='../work/CMORPH_2.50deg-PEN_yyyypp'
      f_fct='../work/FCT_OCN_2.50deg-PEN_yyyypp'

      if( iargc().ge.2 ) then
        f_adj(4:7)='redo'
        f_cmp(4:7)='redo'
        f_fct(4:7)='redo'
      endif

      write(f_fct(29:34), '(i4.4,i2.2)') yyyy, pp

c-----------------------------------------------------------------------
c read in 2.50deg land mask
c-----------------------------------------------------------------------
      open(10, file=f_msk, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_msk
      read(10, rec=1) mask
      close(10)

c-----------------------------------------------------------------------
c initialize arrays for processing window
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          cp(ii,jj)=0
          cr(ii,jj)=0
          sc(ii,jj)=0.0
          sa(ii,jj)=0.0
          sw(ii,jj)=0.0
        enddo
      enddo

c-----------------------------------------------------------------------
c read in adjusted GPCP and CMORPH data for processing window
c-----------------------------------------------------------------------
      do k=-WIN, WIN

        pent=pp+k
        year=yyyy

        if( pent.lt.1 ) then
          pent=pent+NP
          year=yyyy-1
        else if( pent.gt.NP ) then
          pent=pent-NP
          year=yyyy+1
        endif

        write(f_adj(30:35), '(i4.4,i2.2)') year, pent

        write(f_cmp(28:33), '(i4.4,i2.2)') year, pent

        inquire(file=f_adj, exist=adj_there)
        if( .not.adj_there ) then
          write(*, *) '#### file missing: ', f_adj
          stop
        endif
        inquire(file=f_cmp, exist=cmp_there)
        if( .not.cmp_there ) then
          write(*, *) '#### file missing: ', f_cmp
          stop
        endif

        open(11, file=f_adj, status='old',
     1      access='direct', recl=NX*NY*4)
        open(12, file=f_cmp, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_adj
        write(*, *) ' <--  ', f_cmp
        read(11, rec=1) adj
        read(12, rec=1) cmp
        close(11)
        close(12)

        wt=1.0
        do jj=1, NY
          do ii=1, NX
            if( mask(ii,jj).eq.0.0.and.
     1          cmp(ii,jj).ge.0.0.and.
     2          adj(ii,jj).ge.0.0 ) then
              cp(ii,jj)=cp(ii,jj)+1
              sw(ii,jj)=sw(ii,jj)+wt
              sc(ii,jj)=sc(ii,jj)+wt*cmp(ii,jj)
              sa(ii,jj)=sa(ii,jj)+wt*adj(ii,jj)
              if( cmp(ii,jj).ge.0.05 ) then
                cr(ii,jj)=cr(ii,jj)+1
              endif
            endif
          enddo
        enddo

      enddo

c-----------------------------------------------------------------------
c adjust sums over processing window
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          if( mask(ii,jj).eq.0.0.and.sw(ii,jj).gt.0.0 ) then
            sc(ii,jj)=sc(ii,jj)/sw(ii,jj)
            sa(ii,jj)=sa(ii,jj)/sw(ii,jj)
          else
            sc(ii,jj)=-999.0
            sa(ii,jj)=-999.0
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c define adjust factors over ocean
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX

          if( mask(ii,jj).ne.0.0 ) then
            fct(ii,jj)=-999.0
          else
            dis=1
 100        dis2max=dis*dis
            j1=jj-dis
            if( j1.lt.1 ) j1=1
            j2=jj+dis
            if( j2.gt.NY ) j2=NY
            i1=ii-dis
            i2=ii+dis
            ccp=0
            ccr=0
            ssc=0.0
            ssa=0.0
            ssw=0.0
            do j=j1, j2
              do i=i1, i2
                dis2=(j-jj)*(j-jj)+(i-ii)*(i-ii)
                wd=1.0
                if( dis2.le.dis2max ) then
                  js=j
                  if( i.lt.1 ) then
                    is=i+NX
                  else if( i.ge.1.and.i.le.NX ) then
                    is=i
                  else
                    is=i-NX
                  endif
                  if( mask(is,js).eq.0.0 ) then
                    ccp=ccp+cp(is,js)
                    ccr=ccr+cr(is,js)
                    ssw=ssw+wd
                    ssc=ssc+wd*sc(is,js)
                    ssa=ssa+wd*sa(is,js)
                  endif
                endif
              enddo
            enddo
            if( ccp.ge.MIN_PAIR.and.ccr.ge.MIN_RAIN ) then
              fct(ii,jj)=ssa/ssc
            else
              dis=dis+1
              goto 100
            endif
          endif

        enddo
      enddo

c-----------------------------------------------------------------------
c output
c-----------------------------------------------------------------------
      open(20, file=f_fct, access='direct', recl=NX*NY*4)
      write(20, rec=1) fct
      close(20)
      write(*, *) '  ==> ', f_fct

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
